unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, IdBaseComponent, IdComponent, IdTCPServer, StdCtrls,
  IdAntiFreezeBase, IdAntiFreeze, ExtCtrls, IdStack, Menus, ComCtrls,
  IdException;

type
  TForm1 = class(TForm)
    IdTCPServer1: TIdTCPServer;
    Memo1: TMemo;
    IdAntiFreeze1: TIdAntiFreeze;
    Panel1: TPanel;
    HostNameLabel: TLabel;
    HostAddressLabel: TLabel;
    HostName: TLabel;
    HostAddress: TLabel;
    PortLabel: TLabel;
    Port: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    SaveAs1: TMenuItem;
    Exit1: TMenuItem;
    SaveDialog1: TSaveDialog;
    StatusBar1: TStatusBar;
    Edit1: TMenuItem;
    Clear1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    procedure IdTCPServer1Connect(AThread: TIdPeerThread);
    procedure IdTCPServer1Execute(AThread: TIdPeerThread);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure IdTCPServer1Disconnect(AThread: TIdPeerThread);
    procedure FormCreate(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure IdTCPServer1Exception(AThread: TIdPeerThread;
      AException: Exception);
  private
    { Private declarations }
    CounterIP : String;
    function GetFirstToken(S: string; Token: Char): string;
    function StripFirstToken(S: string; Token: Char): string;
    procedure SaveData(Data: string; IPAddr: string);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation
uses
  IdIOHandlerSocket, Unit2;


{$R *.dfm}

function TForm1.GetFirstToken(S: string; Token: Char): string;
var
  Count: Integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count < 1 then
  begin
    GetFirstToken := '';
    Exit;
  end;
  Dec(Count);
  Temp := Copy(S,1,Count);
  GetFirstToken := Temp;
end;

function TForm1.StripFirstToken(S: string; Token: Char): string;
var
  Count: integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count = 0 then begin
    StripFirstToken := S;
    Exit;
  end;
  Temp := S;
  Delete(Temp,1,Count);
  StripFirstToken := Temp;
end;

procedure TForm1.IdTCPServer1Connect(AThread: TIdPeerThread);
begin
  CounterIP := TIdIOHandlerSocket(AThread.Connection.IOHandler).Binding.PeerIP;
  Memo1.Lines.Add('Connected to '+CounterIP);
end;

procedure TForm1.IdTCPServer1Execute(AThread: TIdPeerThread);
var
  DataIn: string;
  IP: string;
begin
  IP := TIdIOHandlerSocket(AThread.Connection.IOHandler).Binding.PeerIP;
  //Memo1.Lines.Add('IP is '+IP);
  try
    DataIn := AThread.Connection.ReadLn(#$A);  // Line Feed terminating character
    //DataIn := AThread.Connection.ReadLn(#$A,10000);  // Time out after 10 seconds
    Memo1.Lines.Add(Trim(DataIn));
    SaveData(DataIn,IP);
    AThread.Connection.WriteLn('ACK,'+GetFirstToken(DataIn,','));
  except
    on E: Exception do
      if (E is EIdConnClosedGracefully) then
        Memo1.Lines.Add(E.Message)
      else
        AThread.Connection.Disconnect;

{
    on E: Exception do
    begin
      //ErrorDialog(E.Message, E.HelpContext);
      Memo1.Lines.Add(E.Message);
    end;
}

  end;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  IdTCPServer1.Active := False;
end;

procedure TForm1.IdTCPServer1Disconnect(AThread: TIdPeerThread);
var
  IP: string;
begin
  IP := TIdIOHandlerSocket(AThread.Connection.IOHandler).Binding.PeerIP;
  Memo1.Lines.Add('Disconnected '+IP);
end;

procedure TForm1.SaveData(Data: string; IPAddr: string);
var
  CurrentDir  : string;
  SaveFile: TextFile;
  CurrentFile: string;
  Date: string;
  Month: string;
  Day: string;
  Year: string;
  CurIP: string;
begin
  GetDir(0,CurrentDir);
  if (IPAddr = '') then
    CurIP := '192.168.2.99'
  else
    CurIP := IPAddr;

  Date := DateToStr(Now);
  Month := GetFirstToken(Date,'/');
  Date := StripFirstToken(Date,'/');
  Day := GetFirstToken(Date,'/');
  Year := StripFirstToken(Date,'/');


  CurrentFile := CurrentDir+'\'+CurIP+'\'+Year+'-'+Month+'-'+Day+'.txt';

  if NOT DirectoryExists(CurrentDir + '\' + CurIP) then
    MkDir(CurrentDir + '\' + CurIP);

  AssignFile(SaveFile, CurrentFile);
  try
    if FileExists(CurrentFile) then
    begin
      Append(SaveFile);
      WriteLn(SaveFile, Data);
    end
    else
    begin
      Rewrite(SaveFile);
      WriteLn(SaveFile, Data);
    end;
  finally
    CloseFile(SaveFile);
  end;

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Form1.Caption := 'UniServe';
  Panel1.Caption := '';
  Memo1.Lines.Clear;
  Memo1.Lines.Add('Recording started on '+DateToStr(Date)+' at '+TimeToStr(Time)+'.');
  HostName.Caption := IdTCPServer1.LocalName;
  HostAddress.Caption := GStack.LocalAddress;
  Port.Caption := IntToStr(IdTCPServer1.DefaultPort);
end;

procedure TForm1.SaveAs1Click(Sender: TObject);
var
  CurrentFile: string;
  SaveFile: TextFile;
begin
  if SaveDialog1.Execute then
  begin
    CurrentFile := SaveDialog1.Filename;
    if Memo1.Lines.Count > 0 then
    begin
      AssignFile(SaveFile, CurrentFile);
      Memo1.Lines.SaveToFile(CurrentFile);
    end;
  end; {if SaveDialog1.Execute}
end;

procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.Clear1Click(Sender: TObject);
begin
  Memo1.Lines.Clear;
end;

procedure TForm1.About1Click(Sender: TObject);
begin
  Form2.Show;
end;

procedure TForm1.IdTCPServer1Exception(AThread: TIdPeerThread;
  AException: Exception);
begin
{
  if (Exception <> EIdConnClosedGracefully) then
    AThread.Connection.Disconnect;
}
end;

end.
