unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Menus, IniFiles, IdBaseComponent,
  IdComponent, IdUDPBase, IdUDPClient, IdAntiFreezeBase, IdAntiFreeze,
  ComCtrls, IdSNTP;

type
  TForm1 = class(TForm)
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Panel1: TPanel;
    Edit1: TEdit;
    IdUDPClient1: TIdUDPClient;
    IdAntiFreeze1: TIdAntiFreeze;
    StatusBar1: TStatusBar;
    Memo1: TMemo;
    Label1: TLabel;
    Label2: TLabel;
    Edit2: TEdit;
    UDP1: TMenuItem;
    ShowTime1: TMenuItem;
    SetTime1: TMenuItem;
    Clear1: TMenuItem;
    SNTP1: TMenuItem;
    RequestSNTP1: TMenuItem;
    IdSNTP1: TIdSNTP;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Edit1Exit(Sender: TObject);
    procedure ShowTime1Click(Sender: TObject);
    procedure SetTime1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure Edit2KeyPress(Sender: TObject; var Key: Char);
    procedure Edit2Exit(Sender: TObject);
    procedure RequestSNTP1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    ReceiveBuffer: string;
    TransmitBuffer: string;
    SNTP_RxBuffer: array[0..128] of byte;
    SNTP_TxBuffer: array[0..128] of byte;
    procedure Communicate;
  public
    { Public declarations }
    function GetPath: string;
    procedure CheckForIni;
    procedure GetIni;
    procedure SetIni;
    function IsIPAddress(zvIPAddressID: string): boolean;
    function IsStrANumber(S: string): Boolean;

    function GetFirstToken(S: string; Token: Char): string;
    function StripFirstToken(S: string; Token: Char): string;
    function Int2StrPad0(N: LongInt; Len: Integer): string;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}
const
  CmdTimeout = 10000;

  NUL = #0;   //null
  SOH = #1;   //start of heading
  STX = #2;   //start of text
  ETX = #3;   //end of text
  EOT = #4;   //end of transmission
  ENQ = #5;   //enquiry
  ACK = #6;   //acknowledge
  BEL = #7;   //bell
  BS  = #8;   //backspace
  HT  = #9;   //horizontal tab
  LF  = #10;  //line feed
  VT  = #11;  //vertical tab
  FF  = #12;  //form feed
  CR  = #13;  //carriage return
  SO  = #14;  //shift out
  SI  = #15;  //shift in
  DLE = #16;  //data link escape
  DC1 = #17;  //device control 1
  DC2 = #18;  //device control 2
  DC3 = #19;  //device control 3
  DC4 = #20;  //device control 4
  NAK = #21;  //negative acknowledge
  SYN = #22;  //synchronous idle
  ETB = #23;  //end of trans. block
  CAN = #24;  //cancel
  EM  = #25;  //end of medium
  SUB = #26;  //substitute
  ESC = #27;  //escape
  FS  = #28;  //file separator
  GS  = #29;  //group separator
  RS  = #30;  //record separator
  US  = #31;  //unit separator
  DEL = #127; //delete

  CRLF = #13+#10;


{Get current directory}
function TForm1.GetPath: string;
var
  Path: string;
  Index: integer;
begin
  Path := ParamStr(0);
  Index := LastDelimiter('\',Path);
  Result := Copy(Path,1,Index);
end;

{Check if ini file already exists. If not, create it}
procedure TForm1.CheckForIni;
var
  F: System.Text;
  Path: string;
begin
  Path := GetPath + 'UDP_Utility.ini';
  if not FileExists(Path) then
  begin
    System.Assign(F, Path);
    System.ReWrite(F);
    System.WriteLn(F, '[IP_Address]');
    System.WriteLn(F, 'CurAddr=192.168.2.4');
    System.Close(F);
  end;
end;

{Read configuration from ini file}
procedure TForm1.GetIni;  {uses IniFiles}
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(GetPath + 'UDP_Utility.ini');
  Edit1.Text := Ini.ReadString('IP_Address','CurAddr','192.168.2.4');
  Edit2.Text := Ini.ReadString('Port','CurPort','6060');
  Ini.Free;
end;

{Update ini file}
procedure TForm1.SetIni;  {uses IniFiles}
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(GetPath + 'UDP_Utility.ini');
  Ini.WriteString('IP_Address', 'CurAddr', Edit1.Text);
  Ini.WriteString('Port','CurPort',Edit2.Text);
  Ini.Free;
end;

{This function return true if the passed string is a true IP address.
 From Internet at http://www.delphi3000.com/articles/article_3458.asp?SK=}
function TForm1.IsIPAddress(zvIPAddressID: string): boolean;
var 
  ipLengthOfGroup, ipPosition, ipNoOfGroups: integer; 
  ipGroupNum, Code: integer; 
begin 
  IsIpAddress := False;
  ipNoOfGroups := 0;
  ipLengthOfGroup := 0;
  for ipPosition := 1 to Length(zvIPAddressID) do
    case Ord(zvIPAddressID[ipPosition]) of
      48..57: 
        begin 
          inc(ipLengthOfGroup); 
          if (ipLengthOfGroup > 3) then exit; 
        end; 
      46: 
        begin 
          inc(ipNoOfGroups); 
          Val(Copy(zvIPAddressID, ipPosition - ipLengthOfGroup, ipLengthOfGroup), ipGroupNum, Code); 
          if ((ipNoOfGroups > 3) or (ipLengthOfGroup = 0)) or (ipGroupNum > 255) then exit; 
          ipLengthOfGroup := 0; 
        end; 
      else 
        exit; 
    end; 
  Val(Copy(zvIPAddressID, ipPosition - ipLengthOfGroup, ipLengthOfGroup), ipGroupNum, Code);
  IsIPAddress := (ipNoOfGroups = 3) and (ipLengthOfGroup > 0) and (ipGroupNum < 256);
end;


function TForm1.IsStrANumber(S: string): Boolean;
var
  P: PChar;
begin
  P      := PChar(S);
  Result := False;
  while P^ <> #0 do
  begin
    if not (P^ in ['0'..'9']) then Exit;
    Inc(P);
  end;
  Result := True;
end;


function TForm1.GetFirstToken(S: string; Token: Char): string;
var
  Count: Integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count < 1 then
  begin
    GetFirstToken := '';
    Exit;
  end;
  Dec(Count);
  Temp := Copy(S,1,Count);
  GetFirstToken := Temp;
end;

function TForm1.StripFirstToken(S: string; Token: Char): string;
var
  Count: integer;
  Temp: string;
begin
  Count := Pos(Token, S);
  if Count = 0 then begin
    StripFirstToken := S;
    Exit;
  end;
  Temp := S;
  Delete(Temp,1,Count);
  StripFirstToken := Temp;
end;

function TForm1.Int2StrPad0(N: LongInt; Len: Integer): string;
var
  S : string;
begin
  Str(N:0,S);
  while Length(S) < Len do
    S := '0' + S;
  Int2StrPad0 := S;
end;

procedure TForm1.Communicate;
begin
  IdUDPClient1.ReceiveTimeout := CmdTimeout;
  IdUDPClient1.Send(TransmitBuffer);
  ReceiveBuffer := IdUDPClient1.ReceiveString();
  if ReceiveBuffer = '' then
  begin
    Memo1.Lines.Add('No response received from the server after ' + IntToStr(IdUDPClient1.ReceiveTimeout div 1000) + ' seconds.');
  end
  else
  begin
    ReceiveBuffer := Trim(ReceiveBuffer);
    Memo1.Lines.Add(ReceiveBuffer);
  end;
end;

//******************************************************************************
procedure TForm1.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CheckForIni;
  GetIni;
  Panel1.Caption := '';
  Memo1.Clear;

  IdUDPClient1.Active := True;
  IdUDPClient1.Host := Edit1.Text;
  IdUDPClient1.Port := StrToInt(Edit2.Text);

  IdSNTP1.Active := True;
  IdSNTP1.Host := Edit1.Text;
end;

procedure TForm1.Edit1Exit(Sender: TObject);
begin
  if IsIPAddress(Edit1.Text) then  // Should have a valid IP address here
  begin
    IdUDPClient1.Host := Edit1.Text;  // Set host IP address from box
    IdSNTP1.Host := Edit1.Text;
    StatusBar1.Panels[0].Text := '';
  end
  else
  begin
    StatusBar1.Font.Color := clRed;
    StatusBar1.Panels[0].Text := 'Invalid IP address';
  end;
end;

procedure TForm1.ShowTime1Click(Sender: TObject);
begin
  TransmitBuffer := 'A'+CRLF; // Show Clock/Calendar
  Communicate;
end;

procedure TForm1.SetTime1Click(Sender: TObject);
var
  CurDate: TDateTime;
  WeekDay: integer;
  MyDate: string;
  DoW: string;
  Month: string;
  Day: string;
  Year: string;
  Time: string;
  Hour: string;
  Minute: string;
  Second: string;
begin
  CurDate := Date;
  WeekDay := DayOfWeek(CurDate);
  DoW := IntToStr(WeekDay);
  if (Length(DoW) < 2) then
    DoW := '0'+DoW;

  MyDate := FormatDateTime('yy/mm/dd', Now);
  Year := GetFirstToken(MyDate,'/');
  MyDate := StripFirstToken(MyDate,'/');
  Month := GetFirstToken(MyDate,'/');
  Day := StripFirstToken(MyDate,'/');

  Time := FormatDateTime('hh:nn:ss', Now);
  Hour := GetFirstToken(Time,':');
  Time := StripFirstToken(Time,':');
  Minute := GetFirstToken(Time,':');
  Second := StripFirstToken(Time,':');

  StatusBar1.Panels[0].Text := ('B,'+Year+','+Month+','+Day+','+DoW+','+Hour+','+Minute+','+Second+LF);
  TransmitBuffer := 'B,'+Year+','+Month+','+Day+','+DoW+','+Hour+','+Minute+','+Second+CRLF;
  Communicate;
end;

procedure TForm1.Clear1Click(Sender: TObject);
begin
  Memo1.Clear;
end;

procedure TForm1.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  if key = Chr(VK_RETURN) then
     Edit2.SetFocus;
end;

procedure TForm1.Edit2KeyPress(Sender: TObject; var Key: Char);
begin
  if key = Chr(VK_RETURN) then
     Edit1.SetFocus;
end;

procedure TForm1.Edit2Exit(Sender: TObject);
begin
  if IsStrANumber(Edit2.Text) = True then  // Should have a valid Port here
  begin
    IdUDPClient1.Port := StrToInt(Edit2.Text);  // Set Port
    StatusBar1.Panels[0].Text := '';
  end
  else
  begin
    StatusBar1.Font.Color := clRed;
    StatusBar1.Panels[0].Text := 'Invalid Port';
  end;
end;

procedure TForm1.RequestSNTP1Click(Sender: TObject);
var
  Flags: Longword;
  RootDelay: Longword;
  RootDispersion: Longword;
  ReferenceID: Longword;
  RefTS1: Longword;
  RefTS2: Longword;
  OriginTS1: Longword;
  OriginTS2: Longword;
  ReceiveTS1: Longword;
  ReceiveTS2: Longword;
  TransmitTS1: Longword;
  TransmitTS2: Longword;

  ReadResult: Integer; // Number of bytes read

begin
  Flags := $DB0011FA;
  RootDelay := $00001708;
  RootDispersion := $0008CA8C;
  ReferenceID := $00000000;
  RefTS1 := $D48B8BE0;
  RefTS2 := $7A814BC4;
  OriginTS1 := $00000000;
  OriginTS2 := $00000000;
  ReceiveTS1 := $00000000;
  ReceiveTS2 := $00000000;
  TransmitTS1 := $D48D79F5;
  TransmitTS2 := $274E1891;

  SNTP_TxBuffer[0] := Flags shr 24;
  SNTP_TxBuffer[1] := Flags shr 16;
  SNTP_TxBuffer[2] := Flags shr 8;
  SNTP_TxBuffer[3] := Flags;

  SNTP_TxBuffer[4] := RootDelay shr 24;
  SNTP_TxBuffer[5] := RootDelay shr 16;
  SNTP_TxBuffer[6] := RootDelay shr 8;
  SNTP_TxBuffer[7] := RootDelay;

  SNTP_TxBuffer[8] := RootDispersion shr 24;
  SNTP_TxBuffer[9] := RootDispersion shr 16;
  SNTP_TxBuffer[10] := RootDispersion shr 8;
  SNTP_TxBuffer[11] := RootDispersion;

  SNTP_TxBuffer[12] := ReferenceID shr 24;
  SNTP_TxBuffer[13] := ReferenceID shr 16;
  SNTP_TxBuffer[14] := ReferenceID shr 8;
  SNTP_TxBuffer[15] := ReferenceID;

  SNTP_TxBuffer[16] := RefTS1 shr 24;
  SNTP_TxBuffer[17] := RefTS1 shr 16;
  SNTP_TxBuffer[18] := RefTS1 shr 8;
  SNTP_TxBuffer[19] := RefTS1;

  SNTP_TxBuffer[20] := RefTS2 shr 24;
  SNTP_TxBuffer[21] := RefTS2 shr 16;
  SNTP_TxBuffer[22] := RefTS2 shr 8;
  SNTP_TxBuffer[33] := RefTS2;

  SNTP_TxBuffer[24] := OriginTS1 shr 24;
  SNTP_TxBuffer[25] := OriginTS1 shr 16;
  SNTP_TxBuffer[26] := OriginTS1 shr 8;
  SNTP_TxBuffer[27] := OriginTS1;

  SNTP_TxBuffer[28] := OriginTS2 shr 24;
  SNTP_TxBuffer[29] := OriginTS2 shr 16;
  SNTP_TxBuffer[30] := OriginTS2 shr 8;
  SNTP_TxBuffer[31] := OriginTS2;

  SNTP_TxBuffer[32] := ReceiveTS1 shr 24;
  SNTP_TxBuffer[33] := ReceiveTS1 shr 16;
  SNTP_TxBuffer[34] := ReceiveTS1 shr 8;
  SNTP_TxBuffer[35] := ReceiveTS1;

  SNTP_TxBuffer[36] := ReceiveTS2 shr 24;
  SNTP_TxBuffer[37] := ReceiveTS2 shr 16;
  SNTP_TxBuffer[38] := ReceiveTS2 shr 8;
  SNTP_TxBuffer[39] := ReceiveTS2;

  SNTP_TxBuffer[40] := TransmitTS1 shr 24;
  SNTP_TxBuffer[41] := TransmitTS1 shr 16;
  SNTP_TxBuffer[42] := TransmitTS1 shr 8;
  SNTP_TxBuffer[43] := TransmitTS1;

  SNTP_TxBuffer[44] := TransmitTS2 shr 24;
  SNTP_TxBuffer[45] := TransmitTS2 shr 16;
  SNTP_TxBuffer[46] := TransmitTS2 shr 8;
  SNTP_TxBuffer[47] := TransmitTS2;

  //IdUDPClient1.ReceiveTimeout := CmdTimeout;
  //IdUDPClient1.SendBuffer(SNTP_TxBuffer,48);

  //IdUDPClient1.ReceiveBuffer(SNTP_TxBuffer,48,5000);

  IdSNTP1.ReceiveTimeout := CmdTimeout;
  IdSNTP1.SendBuffer(SNTP_TxBuffer,48);

  ReadResult := IdSNTP1.ReceiveBuffer(SNTP_RxBuffer,48,5000);
  Memo1.Lines.Add('Read Result: ' + IntToStr(ReadResult));

  Flags := SNTP_RxBuffer[0];
  Flags := (Flags shl 8) + SNTP_RxBuffer[1];
  Flags := (Flags shl 8) + SNTP_RxBuffer[2];
  Flags := (Flags shl 8) + SNTP_RxBuffer[3];

  RootDelay := SNTP_RxBuffer[4];
  RootDelay := (RootDelay shl 8) + SNTP_RxBuffer[5];
  RootDelay := (RootDelay shl 8) + SNTP_RxBuffer[6];
  RootDelay := (RootDelay shl 8) + SNTP_RxBuffer[7];

  RootDispersion := SNTP_RxBuffer[8];
  RootDispersion := (RootDispersion shl 8) + SNTP_RxBuffer[9];
  RootDispersion := (RootDispersion shl 8) + SNTP_RxBuffer[10];
  RootDispersion := (RootDispersion shl 8) + SNTP_RxBuffer[11];
  
  ReferenceID := SNTP_RxBuffer[12];
  ReferenceID := (ReferenceID shl 8) + SNTP_RxBuffer[13];
  ReferenceID := (ReferenceID shl 8) + SNTP_RxBuffer[14];
  ReferenceID := (ReferenceID shl 8) + SNTP_RxBuffer[15];
  
  RefTS1 := SNTP_RxBuffer[16];
  RefTS1 := (RefTS1 shl 8) + SNTP_RxBuffer[17];
  RefTS1 := (RefTS1 shl 8) + SNTP_RxBuffer[18];
  RefTS1 := (RefTS1 shl 8) + SNTP_RxBuffer[19];

  RefTS2 := SNTP_RxBuffer[20];
  RefTS2 := (RefTS2 shl 8) + SNTP_RxBuffer[21];
  RefTS2 := (RefTS2 shl 8) + SNTP_RxBuffer[22];
  RefTS2 := (RefTS2 shl 8) + SNTP_RxBuffer[23];

  OriginTS1 := SNTP_RxBuffer[24];
  OriginTS1 := (OriginTS1 shl 8) + SNTP_RxBuffer[25];
  OriginTS1 := (OriginTS1 shl 8) + SNTP_RxBuffer[26];
  OriginTS1 := (OriginTS1 shl 8) + SNTP_RxBuffer[27];

  OriginTS2 := SNTP_RxBuffer[28];
  OriginTS2 := (OriginTS2 shl 8) + SNTP_RxBuffer[29];
  OriginTS2 := (OriginTS2 shl 8) + SNTP_RxBuffer[30];
  OriginTS2 := (OriginTS2 shl 8) + SNTP_RxBuffer[31];

  ReceiveTS1 := SNTP_RxBuffer[32];
  ReceiveTS1 := (ReceiveTS1 shl 8) + SNTP_RxBuffer[33];
  ReceiveTS1 := (ReceiveTS1 shl 8) + SNTP_RxBuffer[34];
  ReceiveTS1 := (ReceiveTS1 shl 8) + SNTP_RxBuffer[35];

  ReceiveTS2 := SNTP_RxBuffer[36];
  ReceiveTS2 := (ReceiveTS2 shl 8) + SNTP_RxBuffer[37];
  ReceiveTS2 := (ReceiveTS2 shl 8) + SNTP_RxBuffer[38];
  ReceiveTS2 := (ReceiveTS2 shl 8) + SNTP_RxBuffer[39];

  TransmitTS1 := SNTP_RxBuffer[40];
  TransmitTS1 := (TransmitTS1 shl 8) + SNTP_RxBuffer[41];
  TransmitTS1 := (TransmitTS1 shl 8) + SNTP_RxBuffer[42];
  TransmitTS1 := (TransmitTS1 shl 8) + SNTP_RxBuffer[43];

  TransmitTS2 := SNTP_RxBuffer[44];
  TransmitTS2 := (TransmitTS2 shl 8) + SNTP_RxBuffer[45];
  TransmitTS2 := (TransmitTS2 shl 8) + SNTP_RxBuffer[46];
  TransmitTS2 := (TransmitTS2 shl 8) + SNTP_RxBuffer[47];

  Memo1.Lines.Add('Flags: ' + IntToHex(Flags,8));
  Memo1.Lines.Add('RootDelay: ' + IntToHex(RootDelay,8));
  Memo1.Lines.Add('RootDispersion: ' + IntToHex(RootDispersion,8));
  Memo1.Lines.Add('ReferenceID: ' + IntToHex(ReferenceID,8));
  Memo1.Lines.Add('RefTS1: ' + IntToHex(RefTS1,8));
  Memo1.Lines.Add('RefTS2: ' + IntToHex(RefTS2,8));
  Memo1.Lines.Add('OriginTS1: ' + IntToHex(OriginTS1,8));
  Memo1.Lines.Add('OriginTS2: ' + IntToHex(OriginTS2,8));
  Memo1.Lines.Add('ReceiveTS1: ' + IntToHex(ReceiveTS1,8));
  Memo1.Lines.Add('ReceiveTS2: ' + IntToHex(ReceiveTS2,8));
  Memo1.Lines.Add('TransmitTS1: ' + IntToHex(TransmitTS1,8));
  Memo1.Lines.Add('TransmitTS2: ' + IntToHex(TransmitTS2,8));


end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if (IsIPAddress(Edit1.Text) AND IsStrANumber(Edit2.Text)) then
  begin
      SetIni;
      CanClose := true;
  end
  else
      StatusBar1.Panels[0].Text := 'Invalid IP or Port';
end;

end.
